Imports System.Net.Sockets
Imports System.Net
Imports System.Runtime.Serialization.Formatters.Binary

'\\ --[SpoolSocket]-------------------------------------------------------
'\\ Class that communicates to a SpoolMonitorService using sockets 
'\\ ----------------------------------------------------------------------
Public Class SpoolSocket

#Region "Tracing"
    Public Shared SpoolSocketTraceSwitch As New TraceSwitch("SpoolTCPClient", "Printer Monitor Component Tracing")
#End Region

#Region "Private members"
    Private _connected As Boolean
    Private _ServerName As String
    Private _TransportProtocol As String = "tcp"
    Private _Port As Integer = 1912
    Private _Client As Socket
#End Region

#Region "Connect"
    Public Sub Connect(ByVal remoteEP As EndPoint, ByVal client As Socket)

        If SpoolSocketTraceSwitch.TraceVerbose Then
            Trace.WriteLine("Connecting..", Me.GetType.ToString)
        End If
        client.BeginConnect(remoteEP, _
           AddressOf ConnectCallback, client)
    End Sub 'Connect
#End Region

#Region "ConnectCallback"
    Private Sub ConnectCallback(ByVal ar As IAsyncResult)
        If SpoolSocketTraceSwitch.TraceVerbose Then
            Trace.WriteLine("Connected..", Me.GetType.ToString)
        End If
        Try
            ' Retrieve the socket from the state object.
            _Client = CType(ar.AsyncState, Socket)

            ' Complete the connection.
            _Client.EndConnect(ar)

            If SpoolSocketTraceSwitch.TraceVerbose Then
                Trace.WriteLine("Socket connected to {0}", _Client.RemoteEndPoint.ToString())
            End If
            _connected = True

        Catch e As Exception
            If PrinterMonitorComponent.ComponentTraceSwitch.TraceError Then
                Trace.WriteLine(e.ToString, e.Source)
            End If
        End Try
    End Sub 'ConnectCallback
#End Region

#Region "Connected"
    Public ReadOnly Property Connected() As Boolean
        Get
            Return _connected
        End Get
    End Property
#End Region

#Region "Send"
    Private Sub Send(ByVal client As Socket, ByVal data As SpoolMonitoringIPC.SpoolParseRequest)
        ' Convert the data to binary...
        Dim formatter As New BinaryFormatter
        Dim stream As New System.IO.MemoryStream

        If SpoolSocketTraceSwitch.TraceVerbose Then
            Trace.WriteLine("Send : " & data.PrinterName & "," & data.JobId, Me.GetType.ToString)
        End If

        Try
            formatter.Serialize(stream, data)
        Catch e As Exception
            If SpoolSocketTraceSwitch.TraceError Then
                Trace.WriteLine(e.ToString, Me.GetType.ToString)
            End If
        End Try

        Dim byteData As Byte() = stream.GetBuffer

        ' Begin sending the data to the remote device.
        client.BeginSend(byteData, 0, byteData.Length, SocketFlags.None, _
            AddressOf SendCallback, client)


    End Sub 'Send
#End Region

#Region "SendCallback"
    Private Sub SendCallback(ByVal ar As IAsyncResult)
        Try
            ' Retrieve the socket from the state object.
            Dim client As Socket = CType(ar.AsyncState, Socket)

            ' Complete sending the data to the remote device.
            Dim bytesSent As Integer = client.EndSend(ar)
            If SpoolSocketTraceSwitch.TraceVerbose Then
                Trace.WriteLine("Sent " & bytesSent.ToString & " bytes to server.", Me.GetType.ToString)
            End If

        Catch e As Exception
            If PrinterMonitorComponent.ComponentTraceSwitch.TraceError Then
                Trace.WriteLine(e.ToString, e.Source)
            End If
        End Try
    End Sub 'SendCallback

#End Region

#Region "Recieve"
    Private Sub Receive(ByVal client As Socket)

        If PrinterMonitorComponent.ComponentTraceSwitch.TraceVerbose Then
            Trace.WriteLine("Receive..", Me.GetType.ToString)
        End If

        Try
            ' Create the state object.
            Dim state As New SpoolSocketStateObject
            state.workSocket = client

            ' Begin receiving the data from the remote device.
            client.BeginReceive(state.buffer, 0, state.BufferSize, 0, _
                AddressOf ReceiveCallback, state)
        Catch e As Exception
            If PrinterMonitorComponent.ComponentTraceSwitch.TraceError Then
                Trace.WriteLine(e.ToString, e.Source)
            End If
        End Try
    End Sub 'Receive
#End Region

#Region "RecieveCallback"
    Private Sub ReceiveCallback(ByVal ar As IAsyncResult)

        If PrinterMonitorComponent.ComponentTraceSwitch.TraceVerbose Then
            Trace.WriteLine("Received..", Me.GetType.ToString)
        End If

        Try
            ' Retrieve the state object and the client socket 
            ' from the asynchronous state object.
            Dim state As SpoolSocketStateObject = CType(ar.AsyncState, SpoolSocketStateObject)
            Dim client As Socket = state.workSocket

            ' Read data from the remote device.
            Dim bytesRead As Integer = client.EndReceive(ar)

            If bytesRead > 0 Then
                ' There might be more data, so store the data received so far.
                Dim PrevIndex As Integer = state.sbData.Length
                ReDim Preserve state.sbData(state.sbData.Length + bytesRead)
                state.buffer.CopyTo(state.sbData, PrevIndex)

                '  Get the rest of the data.
                client.BeginReceive(state.buffer, 0, state.BufferSize, 0, _
                    AddressOf ReceiveCallback, state)
            Else
                ' All the data has arrived; put it in response.
                If state.sbData.Length > 1 Then
                    ' Turn these bytes into the appropriate object.
                    Dim formatter As New BinaryFormatter
                    Dim stream As New System.IO.MemoryStream(state.sbData)
                    Dim response As SpoolMonitoringIPC.SpoolParseResponse
                    response = CType(formatter.Deserialize(stream), SpoolMonitoringIPC.SpoolParseResponse)
                    If PrinterMonitorComponent.ComponentTraceSwitch.TraceVerbose Then
                        Trace.WriteLine("Recieved data for : " & response.PrinterName & " job : " & response.JobId, Me.GetType.ToString)
                    End If
                End If

            End If
        Catch e As Exception
            Console.WriteLine(e.ToString())
        End Try
    End Sub 'ReceiveCallback
#End Region

#Region "ParseSpoolFile"
    Public Sub ParseSpoolFile(ByVal PrinterName As String, _
                              ByVal JobId As Integer, _
                              ByVal GetTruePageCount As Boolean)

        '\\ Try to send this to the server....
        If SpoolSocketTraceSwitch.TraceVerbose Then
            Trace.WriteLine("ParseSpoolFile(" & PrinterName & "," & JobId.ToString & ")", Me.GetType.ToString)
        End If

        Dim request As New SpoolMonitoringIPC.SpoolParseRequest
        With request
            .PrinterName = PrinterName
            .JobId = JobId
            .GetTruePageCount = GetTruePageCount
        End With
        Dim foo As New SpoolTCPClient(_Port, _ServerName, request)

    End Sub
#End Region

#Region "Constructor"
    Public Sub New(ByVal Server As String)
        _ServerName = Server

        Dim configSettings As New System.Configuration.AppSettingsReader

        Try
            With configSettings
                _Port = Convert.ToInt32(.GetValue("Port", GetType(System.Int32)))
                _TransportProtocol = Convert.ToString(.GetValue("TransportProtocol", GetType(System.String)))
            End With
        Catch e As Exception
            If PrinterMonitorComponent.ComponentTraceSwitch.TraceError OrElse SpoolSocketTraceSwitch.TraceError Then
                Trace.WriteLine(e.ToString)
            End If
        End Try

        '\\ Get the IP address of the server...
        Try
            Dim IPHostEntryServer As IPHostEntry = System.Net.Dns.GetHostEntry(Server)
            Dim IPServerEndpoint As New IPEndPoint(IPHostEntryServer.AddressList(0), _Port)
            _Client = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.IP)
            If SpoolSocketTraceSwitch.TraceVerbose Then
                Trace.WriteLine("Attempting to create a new connection to " & IPServerEndpoint.ToString, Me.GetType.ToString)
            End If
            Call Me.Connect(IPServerEndpoint, _Client)
        Catch e As Exception
            If PrinterMonitorComponent.ComponentTraceSwitch.TraceError Then
                Trace.WriteLine(e.ToString, Me.GetType.ToString)
            End If
        End Try

    End Sub
#End Region

End Class

#Region "SpoolSocketStateObject"
Public Class SpoolSocketStateObject
    ' Client socket.
    Public workSocket As Socket = Nothing
    ' Size of receive buffer.
    Public BufferSize As Integer = 256
    ' Receive buffer.
    Public buffer(256) As Byte
    ' Received data.
    Public sbData() As Byte
End Class 'StateObject

#End Region